# -*- coding: utf-8 -*- 
# @Time    : 03/22/2024 
# @Author  : nzooherd 
# @File    : payment_dao.py 
# @Software: Visual Studio Code


from sqlalchemy import select
from dao import BaseDao
from dao.model import Payment
from exception.insufficient_balance import InsufficientBalanceException


class PaymentDao(BaseDao):

    model = Payment


    @classmethod
    async def modify_amount(cls, user_id: str, amount: int):
        async with cls.session() as session:
            query = select(cls.model).with_for_update(of=cls.model).filter_by(user_id=user_id)
            result = await session.execute(query)
            user_payment = result.scalars().first()
            if user_payment.amount + amount < 0:
                raise InsufficientBalanceException(user_id, user_payment.amount)
            user_payment.amount += amount
            return user_payment
    
    @classmethod
    async def query_by_user_id(cls, user_id: str) -> Payment:
        async with cls.session() as session:
            query = select(cls.model).filter_by(user_id=user_id)
            result = await session.execute(query)
            payment = result.scalars().first()
            if not payment:
                return None
            session.expunge(payment)
            return payment